package top.octopusyan.manager;

import javafx.application.Platform;
import javafx.scene.control.TextArea;
import lombok.Data;
import org.apache.commons.io.FileUtils;
import top.octopusyan.model.ApplicatonStore;
import top.octopusyan.model.ProxySetupModel;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * @author : octopus yan
 * @email : octopus_yan@foxmail.com
 * @description : frp 客户端 工具
 * @create : 2022-4-7 23:19
 */
public class FrpManager {
    public static List<FrpManager> frpcList = new ArrayList<>();
    private static final String YAN_FRP_TEMP_DIR_NAME = "frpclienttmp";
    public static final String FRPC_CLIENT_FILE_NAME = "frpclienttmpfile.exe";
    private final String FRPC_CONF_PREFIX_NAME = "proxy_";
    private final String FRPC_CONF_SUFFIX_NAME = ".ini";
    /**
     * 应用临时目录 地址
     */
    private static final String YAN_FRP_TEMP_DIR_PATH = System.getProperty("java.io.tmpdir") + System.currentTimeMillis() + YAN_FRP_TEMP_DIR_NAME;
    /**
     * frpc 地址
     */
    private static final String FRPC_CLIENT_FILE_PATH = ApplicatonStore.appDataDirPath + File.separator + FRPC_CLIENT_FILE_NAME;
    private static final File frpconfigDir = new File(YAN_FRP_TEMP_DIR_PATH);
    public static final File frpc = new File(FRPC_CLIENT_FILE_PATH);
    public File frpcConfigFile;

    private final ProxySetupModel model;
    private final TextArea console;
    private ProxyThread thread;
    private Process exec;


    private FrpManager(TextArea console, ProxySetupModel model) {
        this.console = console;
        this.model = model;
    }

    public static FrpManager init(ProxySetupModel model) {
        TextArea console = new TextArea();
        console.setPrefHeight(180);
        console.setMinWidth(200);
        FrpManager frpManager = new FrpManager(console, model);
        frpcList.add(frpManager);
        return frpManager;
    }

    public TextArea getConsole() {
        return console;
    }

    /**
     * 开始
     */
    public void start() {
        thread = new ProxyThread();
        thread.start();
    }

    /**
     * 停止隧道
     */
    public void stop() {
        ThreadPoolManager.getInstance().execute(() -> {
            if (thread != null) {
                do {
                    if (exec != null && exec.isAlive())
                        exec.destroy();
                    thread.setExit(true);
                    thread.interrupt();

                } while (!thread.isInterrupted());

                Platform.runLater(() -> {
                    model.setRunning(false);
                    console.appendText("yan-frp-info:已停止\n");
                });

                // 尝试删除缓存配置文件
                if (frpcConfigFile.exists()) frpcConfigFile.delete();
            }
        });
    }

    /**
     * 清理日志
     */
    public void clearLog() {
        if (console != null)
            Platform.runLater(() -> console.clear());
    }

    /**
     * 隧道配置文件地址
     */
    private String getConfigFilePath() {
        return YAN_FRP_TEMP_DIR_PATH + File.separator + FRPC_CONF_PREFIX_NAME + model.getProxyName() + FRPC_CONF_SUFFIX_NAME;
    }

    /**
     * 初始化隧道配置文件
     */
    private void initProxyConfigFile() throws IOException {
        // 创建配置文件
        frpcConfigFile = new File(getConfigFilePath());
        if (!frpcConfigFile.exists()) {
            FileUtils.createParentDirectories(frpcConfigFile);
            frpcConfigFile.createNewFile();
        }
        frpcConfigFile.deleteOnExit();

        // 写入服务配置
        FileUtils.write(frpcConfigFile, ProxyManager.getUserServerConfig(model.get().getNode()), StandardCharsets.UTF_8);
        // 写入隧道配置
        FileUtils.write(frpcConfigFile, ProxyManager.getProxyConfig(model), StandardCharsets.UTF_8, true);
    }

    /**
     * 初始化frpc客户端文件
     */
    public static void initFrpc() throws IOException {
        // frpc 文件
        if (!frpc.exists()) {
            frpc.createNewFile();

            // 复制 frpc
            copyFileUsingFileStreams(FrpManager.class.getResourceAsStream("/static/frpc.exe"), frpc);
        }

        // frpc 临时配置文件
        // 存在就删除
        if (frpconfigDir.exists()) FrpManager.clearTmp();
        // 重新创建
        frpconfigDir.mkdir();
        // 发送删除请求
        frpconfigDir.deleteOnExit();
    }

    /**
     * 清理缓存
     */
    public static void clearTmp() {
        try {
            FileUtils.deleteDirectory(frpconfigDir);

            for (File file : FileUtils.getTempDirectory().listFiles()) {
                if (file.getPath().contains(YAN_FRP_TEMP_DIR_NAME) && file.canWrite() && file.isDirectory()) {
                    FileUtils.deleteDirectory(file);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
            try {
                Runtime.getRuntime().exec("taskkill /im " + FRPC_CLIENT_FILE_NAME + " /f");
                Thread.sleep(1000);
                FileUtils.deleteDirectory(frpconfigDir);
            } catch (InterruptedException | IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    private static void copyFileUsingFileStreams(InputStream input, File dest)
            throws IOException {
        OutputStream output = null;
        try {
            output = new FileOutputStream(dest);
            byte[] buf = new byte[1024];
            int bytesRead;
            while ((bytesRead = input.read(buf)) > 0) {
                output.write(buf, 0, bytesRead);
            }
        } finally {
            input.close();
            output.close();
        }
    }

    @Data
    class ProxyThread extends Thread{
        private volatile boolean exit = false;

        @Override
        public void run() {
            Platform.runLater(() -> console.appendText("yan-frp-info:正在启动\n"));
            try {
                // 检查客户端文件
                if (!frpc.exists()) initFrpc();

                // 初始化配置文件
                initProxyConfigFile();

                // 执行命令
                String command = FRPC_CLIENT_FILE_PATH + " -c " + getConfigFilePath();
                System.out.println(command);
                exec = Runtime.getRuntime().exec(command, null, frpconfigDir);

                // 设置运行状态
                Platform.runLater(() -> model.setRunning(true));

                // 获取标准输出
                BufferedReader readStdout = new BufferedReader(new InputStreamReader(exec.getInputStream()));

                // 逐行读取
                String line;
                while ((line = readStdout.readLine()) != null && !exit) {
                    String finalLine = line;
                    System.out.println("yan-frp-info:" + line);
                    Platform.runLater(() -> console.appendText("yan-frp-info:" + finalLine + "\n"));
                }

            } catch (IOException e) {
                e.printStackTrace();
                //
                Platform.runLater(() -> console.appendText("yan-frp-error:" + e.getMessage() + "\n"));
                Platform.runLater(() -> console.appendText("yan-frp-error:启动失败\n"));
            }
        }
    }
}
